Package: simTool
Type: Package
Title: Conduct Simulation Studies with a Minimal Amount of Source Code
Version: 1.1.9
Authors@R: person("Marsel", "Scheer", role = c("aut", "cre"), email = "scheer@freescience.de")
Maintainer: Marsel Scheer <scheer@freescience.de>
Description: Tool for statistical simulations that have two components. 
    One component generates the data and the other one
    analyzes the data. The main aims of the package are the reduction
    of the administrative source code (mainly loops and management code for the
    results) and a simple applicability of the package that allows the user to
    quickly learn how to work with it. Parallel computing is
    also supported. Finally, convenient functions are provided to summarize the
    simulation results.
Depends: R (>= 2.14.0)
Imports: dplyr (>= 0.7.2), purrr (>= 0.2.3), tidyr (>= 1.0.0), tibble
        (>= 2.0.0), vctrs (>= 0.3.0), parallel, methods
Suggests: ggplot2, knitr, boot, broom, rmarkdown, tinytest, lintr,
        roxygen2, covr
License: GPL-3
VignetteBuilder: knitr
RoxygenNote: 7.3.3
URL: https://marselscheer.github.io/simTool/
BugReports: https://github.com/MarselScheer/simTool/issues
NeedsCompilation: no
Packaged: 2026-01-24 11:04:25 UTC; rstudio
Author: Marsel Scheer [aut, cre]
Repository: CRAN
Date/Publication: 2026-01-24 11:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 21:39:19 UTC; unix
